cann/asc-devkit bfloat16高低位交换函数
__lowhigh2highlow
【免费下载链接】asc-devkit本项目是CANN 推出的昇腾AI处理器专用的算子程序开发语言,原生支持C和C++标准规范,主要由类库和语言扩展层构成,提供多层级API,满足多维场景算子开发诉求。项目地址: https://gitcode.com/cann/asc-devkit
产品支持情况
- Ascend 950PR/Ascend 950DT:支持
- Atlas A3 训练系列产品/Atlas A3 推理系列产品:不支持
- Atlas A2 训练系列产品/Atlas A2 推理系列产品:不支持
- Atlas 200I/500 A2 推理产品:不支持
- Atlas 推理系列产品AI Core:不支持
- Atlas 推理系列产品Vector Core:不支持
- Atlas 训练系列产品:不支持
功能说明
将输入数据的高低16位进行交换并返回。
函数原型
inline bfloat16x2_t __lowhigh2highlow(const bfloat16x2_t x)参数说明
表1参数说明
| 参数名 | 输入/输出 | 描述 |
|---|---|---|
| x | 输入 | 源操作数。 |
返回值说明
输入数据的高低16位进行交换的结果。特殊值如下:
| x值 | 返回值 |
|---|---|
| (0, -0) | (-0, 0) |
| (nan, 正常值) | (正常值, nan) |
| (inf, -inf) | (-inf, inf) |
| (ASCRT_MAX_NORMAL_BF16, -ASCRT_MAX_NORMAL_BF16) | (-ASCRT_MAX_NORMAL_BF16, ASCRT_MAX_NORMAL_BF16) |
| (nan, nan) | (nan, nan) |
约束说明
无
需要包含的头文件
使用该接口需要包含"simt_api/asc_bf16.h"头文件。
#include "simt_api/asc_bf16.h"调用示例
SIMT编程场景:
// 使用短向量可提升数据搬运效率 __aicore__ void simt_lowhigh2highlow(bfloat16x2_t* input, bfloat16x2_t* output, uint32_t input_total_length) { uint32_t idx = blockIdx.x * blockDim.x + threadIdx.x; // 每个线程处理1个bfloat16x2_t类型的数据,即2个bfloat16_t类型的数据,因此idx >= input_total_length / 2的线程不处理数据 if (idx > input_total_length / 2) { return; } output[idx] = __lowhigh2highlow(input[idx]); } __global__ __launch_bounds__(1024) void cast_kernel(bfloat16_t* input, bfloat16_t* output, uint32_t input_total_length) { simt_lowhigh2highlow((bfloat16x2_t*)input, (bfloat16x2_t*)output, input_total_length); }SIMD与SIMT混合编程场景:
// 使用短向量可提升数据搬运效率 __simt_vf__ __launch_bounds__(1024) inline void simt_lowhigh2highlow(__gm__ bfloat16x2_t* input, __gm__ bfloat16x2_t* output, uint32_t input_total_length) { uint32_t idx = blockIdx.x * blockDim.x + threadIdx.x; // 每个线程处理1个bfloat16x2_t类型的数据,即2个bfloat16_t类型的数据,因此idx >= input_total_length / 2的线程不处理数据 if (idx > input_total_length / 2) { return; } output[idx] = __lowhigh2highlow(input[idx]); } __global__ __vector__ void cast_kernel(__gm__ bfloat16_t* input, __gm__ bfloat16_t* output, uint32_t input_total_length) { asc_vf_call<simt_lowhigh2highlow>(dim3(1024), (__gm__ bfloat16x2_t*)input, (__gm__ bfloat16x2_t*)output, input_total_length); }
【免费下载链接】asc-devkit本项目是CANN 推出的昇腾AI处理器专用的算子程序开发语言,原生支持C和C++标准规范,主要由类库和语言扩展层构成,提供多层级API,满足多维场景算子开发诉求。项目地址: https://gitcode.com/cann/asc-devkit
创作声明:本文部分内容由AI辅助生成(AIGC),仅供参考